import argparse
import distutils.dir_util
import os
import subprocess
import sys
import tempfile


def call_process(call_args, custom_error, exit_on_error=False):
    try:
        subprocess.check_output(call_args)
    except subprocess.CalledProcessError as e:
        print(e.output)
        print('{}. Return code: {}'.format(custom_error, e.returncode))
        if exit_on_error:
            sys.exit(1)
        else:
            raise


parser = argparse.ArgumentParser(description='Publishes HawkTracer documentation')
parser.add_argument('-b', '--build-dir', help='HawkTracer build directory', required=True)
parser.add_argument('-d', '--publish-dirs', nargs='+',
                    help='Directories for this version of the documentation, e.g. 0.7.0, stable etc.')
parser.add_argument('-r', '--repository', help='HawkTracer repository path',
                    default='git@github.com:hawktracer/doc.git')
args = parser.parse_args()

call_process(['cmake', '--build', args.build_dir, '--target', 'doc_doxygen'], 'Building documentation failed.', True)

doc_path = os.path.abspath(os.path.join(args.build_dir, 'doxygen_doc', 'html'))
if not os.path.exists(doc_path):
    print('Documentation path "{}" doesn\'t exist!'.format(doc_path))
    sys.exit(1)

current_dir = os.getcwd()
clone_dir = tempfile.mkdtemp()

try:
    call_process(['git', 'clone', args.repository, clone_dir], 'Unable to clone repository')
    os.chdir(clone_dir)
    call_process(['git', 'checkout', 'gh-pages'], 'Unable to switch to doc branch.')

    print('Removing old documentation...')
    for publish_dir in args.publish_dirs:
        publish_dir = os.path.join(clone_dir, publish_dir)
        if os.path.isdir(publish_dir):
            call_process(['git', 'rm', '-rfq', publish_dir], 'Unable to remove previous documentation')

        print('Copying documentation to a repository...')
        distutils.dir_util.copy_tree(doc_path, publish_dir)

    print("Committing documentation...")
    call_process(['git', 'add', '.'], 'Unable to add files to commit.')
    call_process(['git', 'commit', '-a', '-m', 'update doc'], 'Unable to commit new documentation.')
    call_process(['git', 'push', 'origin', 'gh-pages'], 'Unable to push documentation.')

finally:
    distutils.dir_util.remove_tree(clone_dir)



#!/usr/bin/env python
#===- lib/fuzzer/scripts/unbalanced_allocs.py ------------------------------===#
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
#===------------------------------------------------------------------------===#
#
# Post-process -trace_malloc=2 output and printout only allocations and frees
# unbalanced inside of fuzzer runs.
# Usage:
#   my_fuzzer -trace_malloc=2 -runs=10 2>&1 | unbalanced_allocs.py -skip=5
#
#===------------------------------------------------------------------------===#

import argparse
import sys

_skip = 0

def PrintStack(line, stack):
  global _skip
  if _skip > 0:
    return
  print('Unbalanced ' + line.rstrip());
  for l in stack:
    print(l.rstrip())

def ProcessStack(line, f):
  stack = []
  while line and line.startswith('    #'):
    stack += [line]
    line = f.readline()
  return line, stack

def ProcessFree(line, f, allocs):
  if not line.startswith('FREE['):
    return f.readline()

  addr = int(line.split()[1], 16)
  next_line, stack = ProcessStack(f.readline(), f)
  if addr in allocs:
    del allocs[addr]
  else:
    PrintStack(line, stack)
  return next_line

def ProcessMalloc(line, f, allocs):
  if not line.startswith('MALLOC['):
    return ProcessFree(line, f, allocs)

  addr = int(line.split()[1], 16)
  assert not addr in allocs

  next_line, stack = ProcessStack(f.readline(), f)
  allocs[addr] = (line, stack)
  return next_line

def ProcessRun(line, f):
  if not line.startswith('MallocFreeTracer: START'):
    return ProcessMalloc(line, f, {})

  allocs = {}
  print(line.rstrip())
  line = f.readline()
  while line:
    if line.startswith('MallocFreeTracer: STOP'):
      global _skip
      _skip = _skip - 1
      for _, (l, s) in allocs.items():
        PrintStack(l, s)
      print(line.rstrip())
      return f.readline()
    line = ProcessMalloc(line, f, allocs)
  return line

def ProcessFile(f):
  line = f.readline()
  while line:
    line = ProcessRun(line, f);

def main(argv):
  parser = argparse.ArgumentParser()
  parser.add_argument('--skip', default=0, help='number of runs to ignore')
  args = parser.parse_args()
  global _skip
  _skip = int(args.skip) + 1
  ProcessFile(sys.stdin)

if __name__ == '__main__':
  main(sys.argv)



import sys
import subprocess
import re


class AuthorInfo:
    added_lines = 0
    removed_lines = 0
    changed_files = 0
    name = ''
    email = ''
    commit_count = 0

    def __init__(self, added_lines, removed_lines, changed_files, name, email, commit_count):
        self.added_lines = added_lines
        self.removed_lines = removed_lines
        self.changed_files = changed_files
        self.name = name
        self.email = email
        self.commit_count = commit_count


class GitInfo:
    _from_version = ''
    _to_version = ''

    def __init__(self, from_version, to_version):
        self._from_version = from_version
        self._to_version = to_version

    def get_summary_stats(self):
        return subprocess.getoutput('git diff {}..{} --shortstat'.format(self._from_version, self._to_version)).strip()

    def get_number_of_commits(self):
        return len(subprocess.getoutput('git log {}..{} --oneline'
                                        .format(self._from_version, self._to_version)).splitlines())

    def _get_author_stats(self, author_name, author_email, commit_count):
        stats = subprocess.getoutput(
            'git log {}..{} --numstat --pretty="" --author="{} <{}>"'.format(
                self._from_version, self._to_version, author_name, author_email)).splitlines()

        added_lines = 0
        removed_lines = 0
        for stat_line in stats:
            try:
                added_lines = added_lines + int(stat_line.split()[0])
                removed_lines = removed_lines + int(stat_line.split()[1])
            except ValueError:
                pass

        return AuthorInfo(added_lines, removed_lines, len(stats), author_name, author_email, commit_count)

    def get_list_of_authors(self):
        author_regex = re.compile('(\d+)\s+(.+)\s+<(.+)>')
        git_authors = subprocess.getoutput('git shortlog {}..{} -sne'
                                           .format(self._from_version, self._to_version)).splitlines()
        return [self._get_author_stats(m.group(2), m.group(3), m.group(1))
                for l in git_authors for m in [author_regex.search(l)] if m]

    @staticmethod
    def get_most_recent_tags():
        tag_regex = re.compile('tag:\s(v[a-zA-Z\d.]+)')
        git_tags = subprocess.getoutput('git log  --tags --simplify-by-decoration --pretty="format:%D"').splitlines()
        return [m.group(1) for l in git_tags for m in [tag_regex.search(l)] if m][0:2]

    @staticmethod
    def is_master_branch():
        return subprocess.getoutput('git rev-parse --abbrev-ref HEAD').strip() == 'master'


if len(sys.argv) < 2:
    print('Usage: {} <codename>'.format(sys.argv[0]))
    sys.exit(1)

if not GitInfo.is_master_branch():
    print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
    print('!!! YOU ARE NOT IN MASTER BRANCH !!!')
    print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
    print('')

tags = GitInfo.get_most_recent_tags()

if len(tags) < 2:
    print('Expected to find at least 2 git tags, but found {}'.format(len(tags)))
    sys.exit(1)

current_tag = tags[0]
current_version = current_tag[1:]
previous_tag = tags[1]
previous_version = previous_tag[1:]
code_name = sys.argv[1]

git_info = GitInfo(previous_tag, current_tag)

print("I'm pleased to announce HawkTracer {} ({}) release!".format(current_version, code_name))
print("URL: https://github.com/amzn/hawktracer/releases/tag/{}".format(current_tag))
print("Changes")
print("==========================================")
found_version = False
with open("NEWS") as f:
    for line in f:
        if line.startswith(current_version):
            found_version = True
        elif line.startswith(previous_version):
            break
        elif found_version:
            sys.stdout.write(line)

authors = git_info.get_list_of_authors()

print("Git statistics")
print("==========================================")
print("  {}".format(git_info.get_summary_stats()))
print("  Number of commits: {}".format(git_info.get_number_of_commits()))
print("  Contributors: {}".format(len(authors)))
for author in authors:
    alignment = len(authors[0].commit_count) - len(author.commit_count)
    print("    * {}{} {} <{}> {} files changed (+{}, -{})".format(
        ' ' * alignment, author.commit_count, author.name, author.email,
        author.changed_files, author.added_lines, author.removed_lines))



"""Example python3 application that uses HawkTracer tracing system.

This application demonstrates how to instrument python3 code to measure
execution time of a specific function.

This is advanced example that demonstrates all the existing API usage.
See instrumentation-base-example.py for the simplest possible instrumentation
example.
"""

import hawktracer.core

hawktracer.core.init()

# Enable decorators, so @hawktracer.core.trace annotation has an effect
hawktracer.core.enable_trace_decorator()

# Enable tracing. This can also be enabled by setting HT_PYTHON_TRACE_ENABLED environment variable.
hawktracer.core.enable_tracing()

@hawktracer.core.trace
def foo():
    print("This is foo.")
    print("This function is traced, as it has @trace decorator.")

hawktracer.core.disable_trace_decorator()

def bar():
    print("This is bar.")
    print("This function isn't traced, as it doesn't have @trace decorator.")

@hawktracer.core.trace
def foobar():
    foo()
    print("This is foobar")
    print("This function isn't traced even though @trace decorator is added to this function,")
    print("but disable_trace_decorator() was called first.")

hawktracer.core.register_file_listener("test.htdump", 1024*1024);

for i in range(10):
    if i % 2 == 0:
        hawktracer.core.disable_tracing() # temporarily disable tracing

    foo()

    if i % 2 == 0:
        hawktracer.core.enable_tracing()

    bar()
    foobar()




"""Example HawkTracer client

This application demonstrates how to implement simple
HawkTracer client using hawktracer.client.Client class.
This application visualizes selected event's field values
on a graph.
"""

import argparse
import hawktracer.client
import matplotlib.pyplot as plt
import os
import time

example_usage = '''
Example:
    python {} -s 127.0.0.1:8765 -e HT_CallstackIntEvent -f duration
'''.format(os.path.basename(__file__))

parser = argparse.ArgumentParser(description='Draw selected field from the event\'s klass on a graph.',
                                 epilog=example_usage,
                                 formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument('-s', '--source', help='Description of the source; either file name or IP address', required=True)
parser.add_argument('-e', '--klass', help='Klass of event to draw on the graph', required=True)
parser.add_argument('-f', '--field', help='A field of event\'s klass to draw on the graph', required=True)
args = parser.parse_args()

client = hawktracer.client.Client()
client.start(args.source)

data = []
was_eos = False

while True:
    event = client.poll_event()
    if event:
        if event[0] == args.klass and args.field in event[1]:
            data.append(event[1][args.field])
        else:
            continue
    else:
        if was_eos:
            break
        was_eos = client.eos()
        time.sleep(0.1)
        continue

    length = len(data)
    plt.xlim(length - 10, length)
    plt.plot(list(range(1, length+1)), data)
    plt.pause(0.05)
